@@ -246,7 +246,6 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
   JSCValue *js_ephy;
   JSCValue *js_serializer;
   JSCValue *js_result;
-  gboolean remember_passwords;
   guint i;
 
   frame = webkit_web_page_get_main_frame (web_page);
@@ -265,14 +264,11 @@ web_page_form_controls_associated (WebKitWebPage    *web_page,
                                           G_CALLBACK (sensitive_form_message_serializer), NULL, NULL,
                                           G_TYPE_STRING, 2,
                                           G_TYPE_UINT64, G_TYPE_BOOLEAN);
-  remember_passwords = !extension->is_private_profile &&
-                       g_settings_get_boolean (EPHY_SETTINGS_WEB_EXTENSION_WEB, EPHY_PREFS_WEB_REMEMBER_PASSWORDS);
   js_result = jsc_value_object_invoke_method (js_ephy,
                                               "formControlsAssociated",
                                               G_TYPE_UINT64, webkit_web_page_get_id (web_page),
                                               G_TYPE_PTR_ARRAY, form_controls,
                                               JSC_TYPE_VALUE, js_serializer,
-                                              G_TYPE_BOOLEAN, remember_passwords,
                                               G_TYPE_NONE);
   g_object_unref (js_result);
   g_ptr_array_unref (form_controls);
@@ -650,6 +646,14 @@ js_is_edited (JSCValue *js_element)
   return webkit_dom_element_html_input_element_is_user_edited (WEBKIT_DOM_ELEMENT (node));
 }
 
+static gboolean
+js_should_remember_passwords (EphyWebExtension *extension)
+{
+  g_assert (EPHY_IS_WEB_EXTENSION (extension));
+
+  return !extension->is_private_profile && g_settings_get_boolean (EPHY_SETTINGS_WEB_EXTENSION_WEB, EPHY_PREFS_WEB_REMEMBER_PASSWORDS);
+}
+
 static void
 js_exception_handler (JSCContext   *context,
                       JSCException *exception)
@@ -740,12 +744,10 @@ window_object_cleared_cb (WebKitScriptWorld *world,
                                                  js_ephy);
 
   if (!extension->is_private_profile) {
-    guint64 page_id = webkit_web_page_get_id (page);
-    g_assert (page_id < G_MAXINT32);
-
     g_autoptr(JSCValue) js_password_manager_ctor = jsc_value_object_get_property (js_ephy, "PasswordManager");
     g_autoptr(JSCValue) js_password_manager = jsc_value_constructor_call (js_password_manager_ctor,
-                                                                          G_TYPE_INT, page_id, G_TYPE_NONE);
+                                                                          G_TYPE_UINT64, webkit_web_page_get_id (page),
+                                                                          G_TYPE_NONE);
     jsc_value_object_set_property (js_ephy, "passwordManager", js_password_manager);
 
     js_function = jsc_value_new_function (js_context,
@@ -773,6 +775,14 @@ window_object_cleared_cb (WebKitScriptWorld *world,
   jsc_value_object_set_property (js_ephy, "isEdited", js_function);
   g_object_unref (js_function);
 
+  js_function = jsc_value_new_function (js_context,
+                                        "shouldRememberPasswords",
+                                        G_CALLBACK (js_should_remember_passwords),
+                                        g_object_ref (extension), g_object_unref,
+                                        G_TYPE_BOOLEAN, 0);
+  jsc_value_object_set_property (js_ephy, "shouldRememberPasswords", js_function);
+  g_object_unref (js_function);
+
   g_object_unref (js_ephy);
   g_object_unref (js_context);
 }
