@@ -1600,13 +1600,14 @@ create_language_section (PrefsDialog *dialog)
       g_autofree char *normalized_locale = normalize_locale (code);
       if (normalized_locale != NULL) {
         g_autofree char *language_name = language_for_locale (normalized_locale);
+        if (language_name == NULL)
+          language_name = g_strdup (normalized_locale);
         language_editor_add (dialog, normalized_locale, language_name);
       }
     }
   }
   g_object_unref (store);
 
-  language_editor_update_buttons (dialog);
   g_strfreev (list);
 
   /* Lockdown if key is not writable */
@@ -1625,6 +1626,8 @@ create_language_section (PrefsDialog *dialog)
   g_settings_bind_writable (EPHY_SETTINGS_WEB,
                             EPHY_PREFS_WEB_LANGUAGE,
                             dialog->lang_treeview, "sensitive", FALSE);
+
+  language_editor_update_buttons (dialog);
 }
 
 static void
